/**
 * EasyBeans
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbstractJNDIInjectionHandler.java 4618 2009-02-17 14:47:24Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.annotation.processor.handler;

import org.ow2.util.annotation.processor.ProcessorException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * An {@link AbstractJNDIInjectionHandler} is an injection
 * {@link org.ow2.util.annotation.processor.IAnnotationHandler} that provides
 * facility for injecting values retrieved from JNDI.
 * @author Guillaume Sauthier
 */
public abstract class AbstractJNDIInjectionHandler extends AbstractInjectionHandler {

    /**
     * <code>java:comp/env</code> {@link Context}.
     */
    private Context envContext;

    /**
     * Construct a new {@link AbstractJNDIInjectionHandler} using the given
     * <code>java:comp/env</code> {@link Context} provided.
     * @param envContext <code>java:comp/env</code> {@link Context}
     */
    public AbstractJNDIInjectionHandler(final Context envContext) {
        this.envContext = envContext;
    }

    /**
     * Perform a lookup operation directly on the registry (global JNDI). Notice
     * that this method do not lookup in the <code>java:comp/env</code>
     * {@link Context}.
     * @param name name of the resource to be looked up
     * @return the Object returned from the lookup
     * @throws ProcessorException if lookup
     *         fail
     */
    protected Object doRegistryLookup(final String name) throws ProcessorException {
        InitialContext ictx;
        try {
            ictx = new InitialContext();
            return ictx.lookup(name);
        } catch (NamingException e) {
            throw new ProcessorException("Cannot lookup '" + name + "'", e);
        }

    }

    /**
     * Perform a lookup operation on the <code>java:comp/env</code>
     * {@link Context}.
     * @param name name of the resource to be looked up
     * @return the Object returned from the lookup
     * @throws ProcessorException if lookup
     *         fail
     */
    protected Object doContextLookup(final String name) throws ProcessorException {
        try {
            return envContext.lookup(name);
        } catch (NamingException e) {
            throw new ProcessorException("Cannot lookup 'java:comp/env/" + name + "'", e);
        }
    }

    /**
     * Compute the standard default JNDI name in which a {@link Field} should
     * lookup its value.
     * @param field original {@link Field}
     * @return the standard default JNDI name
     */
    protected String getStandardBindingName(final Field field) {
        Class<?> support = field.getDeclaringClass();
        String jndiName = support.getName();

        // Directly use field's name as attribute name
        jndiName = jndiName.concat("/").concat(field.getName());
        return jndiName;
    }

    /**
     * Compute the standard default JNDI name in which a {@link Method} should
     * lookup its value.
     * @param method original {@link Method}
     * @return the standard default JNDI name
     */
    protected String getStandardBindingName(final Method method) throws ProcessorException {
        Class<?> support = method.getDeclaringClass();
        String jndiName = support.getName();

        // Extract attribute's name from the method
        jndiName = jndiName.concat("/").concat(getAttributeName(method));
        return jndiName;
    }

}
